@@ -284,7 +284,7 @@ class CopyDetector:
         # before run() is called, similarity data should be empty
         self.similarity_matrix = np.array([])
         self.token_overlap_matrix = np.array([])
-        self.slice_matrix = np.array([])
+        self.slice_matrix = {}
         self.file_data = {}
 
     @staticmethod
@@ -494,12 +494,15 @@ class CopyDetector:
             print("  0.00: Generating file fingerprints")
         self._preprocess_code(self.test_files + self.ref_files)
 
-        self.similarity_matrix = np.full((
-            len(self.test_files),len(self.ref_files),2), -1, dtype=np.float64)
-        self.token_overlap_matrix = np.full((
-            len(self.test_files), len(self.ref_files)), -1)
-        self.slice_matrix = [[np.array([]) for _ in range(len(self.ref_files))]
-                             for _ in range(len(self.test_files))]
+        self.similarity_matrix = np.full(
+            (len(self.test_files), len(self.ref_files), 2),
+            -1,
+            dtype=np.float64,
+        )
+        self.token_overlap_matrix = np.full(
+            (len(self.test_files), len(self.ref_files)), -1
+        )
+        self.slice_matrix = {}
 
         if not self.silent:
             print(f"{time.time()-start_time:6.2f}: Beginning code comparison")
@@ -525,15 +528,16 @@ class CopyDetector:
                     ref_idx, test_idx = comparisons[(ref_f, test_f)]
                     overlap = self.token_overlap_matrix[ref_idx, test_idx]
                     sim2, sim1 = self.similarity_matrix[ref_idx, test_idx]
-                    slices2, slices1 = self.slice_matrix[ref_idx][test_idx]
                 else:
                     overlap, (sim1, sim2), (slices1, slices2) = compare_files(
-                        self.file_data[test_f], self.file_data[ref_f])
+                        self.file_data[test_f], self.file_data[ref_f]
+                    )
                     comparisons[(test_f, ref_f)] = (i, j)
+                    if slices1.shape[0] != 0:
+                        self.slice_matrix[(i, j)] = [slices1, slices2]
 
-                self.similarity_matrix[i,j] = np.array([sim1, sim2])
-                self.slice_matrix[i][j] = [slices1, slices2]
-                self.token_overlap_matrix[i,j] = overlap
+                self.similarity_matrix[i, j] = np.array([sim1, sim2])
+                self.token_overlap_matrix[i, j] = overlap
 
         if not self.silent:
             print(f"{time.time()-start_time:6.2f}: Code comparison completed")
@@ -587,8 +591,12 @@ class CopyDetector:
 
             test_sim = self.similarity_matrix[x[idx], y[idx], 0]
             ref_sim = self.similarity_matrix[x[idx], y[idx], 1]
-            slices_test = self.slice_matrix[x[idx]][y[idx]][0]
-            slices_ref = self.slice_matrix[x[idx]][y[idx]][1]
+            if (x[idx], y[idx]) in self.slice_matrix:
+                slices_test = self.slice_matrix[(x[idx], y[idx])][0]
+                slices_ref = self.slice_matrix[(x[idx], y[idx])][1]
+            else:
+                slices_test = self.slice_matrix[(y[idx], x[idx])][1]
+                slices_ref = self.slice_matrix[(y[idx], x[idx])][0]
 
             if self.truncate:
                 truncate = 10
@@ -650,7 +658,7 @@ class CopyDetector:
         plt.close()
 
         # render template with jinja and save as html
-        with open(data_dir + "report.html") as template_fp:
+        with open(data_dir + "report.html", encoding="utf-8") as template_fp:
             template = Template(template_fp.read())
 
         flagged = self.similarity_matrix[:,:,0] > self.display_t
@@ -664,7 +672,7 @@ class CopyDetector:
                                  sim_hist_base64=sim_hist_base64)
 
         if output_mode == "save":
-            with open(self.out_file, "w") as report_f:
+            with open(self.out_file, "w", encoding="utf-8") as report_f:
                 report_f.write(output)
 
             if not self.silent:
